--[[
Plugin that handles filtering of chat and raid warnings
]]

----------------------------------
-- Create plugin

local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:Plugin("Filter")
if not plugin then return end

----------------------------------
-- Locals 

local L = LibStub("AceLocale-3.0"):GetLocale("RW2-Plugin-Filter")
local LB = LibStub("AceLocale-3.0"):GetLocale("RW2-Plugin-Base")

local db
local Utils = RW.Utils
local inCombat = nil
local party
local whisperSpamm 

local defaults = {
	profile = {
		PluginEnabled = true,
		WhisperReply = true,
		BossModBlock = true,
		BossEmotesFrame = false,
		BossEmoteChat = false,
		FilterOutIn = true,
	}
}

----------------------------------
-- Init

function plugin:OnRegister()
	self.db = RW.db:RegisterNamespace("Filter", defaults)
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Reset")
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")
end

function plugin:PluginEnable()
	if not db.PluginEnabled then
		self:Disable()
		return
	end
	
	self:RegisterMessage("RW_BossEngaged")
	self:RegisterMessage("RW_BossEnded")
	party = RW:GetPlugin("Party")
	whisperSpamm = {}
	
	self:Apply()
end

function plugin:PluginDisable()
	self:Remove()
end

function plugin:Reset()
	db = self.db.profile
end

function plugin:RW_BossEngaged(message, boss)
	inCombat = boss
	if db.WhisperReply then
		self:RegisterEvent("CHAT_MSG_WHISPER")
	end
end

function plugin:RW_BossEnded(message, boss)
	inCombat = nil
	self:UnregisterEvent("CHAT_MSG_WHISPER")
	wipe(whisperSpamm)
end

local function removeSpamm(name)
	whisperSpamm[name] = nil
end


function plugin:CHAT_MSG_WHISPER(event, msg, sender)
	if inCombat and msg == "status" then
		SendChatMessage(L.RW_CHAT_PREFIX ..L.RW_CHAT_RESPONSE:format(
			RW.LBB[inCombat:GetName()] or inCombat:GetName(), 
			inCombat:GetBossHealth() * 100,
			Utils:TimeToStringShort(GetTime() - inCombat.engageTime),
			party:GetNumAlive(), 
			party:GetPartySize()), "WHISPER", nil, sender)
			
	elseif inCombat and not whisperSpamm[sender] and not party.group[sender] then
		whisperSpamm[sender] = true
		SendChatMessage(L.RW_CHAT_PREFIX ..L.RW_CHAT_RESPONSE:format(
			RW.LBB[inCombat:GetName()] or inCombat:GetName(), 
			inCombat:GetBossHealth() * 100, 
			Utils:TimeToStringShort(GetTime() - inCombat.engageTime),
			party:GetNumAlive(), 
			party:GetPartySize()), "WHISPER", nil, sender)
		inCombat:Schedule("removeSpamm", 10, removeSpamm, sender)
	end
end

local function filterWhisperOut(self, event, ...)
	local msg = ...
	if not msg then return end
	return msg:sub(1, L.RW_CHAT_PREFIX:len()) == L.RW_CHAT_PREFIX, ...
end
	
local function filterWhisperIn(self, event, ...)
	local msg = ...
	if not msg then return false, ... end
	return inCombat and msg == "status", ...
end

local function filterRaidWarning(self, event, ...)
	local msg = ...
	if not msg then return false, ... end
	return msg:match("%*%*%* .* %*%*%*"), ...
end

local function filterBossEmote(self, event, ...)
	return (inCombat and true or false), ...
end

function plugin:Apply()
	if db.BossModBlock then
		ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID_WARNING", filterRaidWarning)
	end
	if db.BossEmotesChat then
		ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID_BOSS_EMOTE", filterBossEmote)
	end
	ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", filterWhisperIn)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", filterWhisperOut)
end

function plugin:Remove()
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_RAID_WARNING", filterRaidWarning)
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_WHISPER", filterWhisperIn)
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_WHISPER_INFORM", filterWhisperOut)
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_RAID_BOSS_EMOTE", filterBossEmote)
end

local oldRW = RaidWarningFrame:GetScript("OnEvent")
RaidWarningFrame:SetScript("OnEvent", function(self, event, msg, ...)
	if db.BossModBlock and inCombat then 
		if msg:match("%*%*%* .* %*%*%*") then
			return
		end
	end
	return oldRW(self, event, msg, ...)
end)

local oldBE = RaidBossEmoteFrame:GetScript("OnEvent")
RaidBossEmoteFrame:SetScript("OnEvent", function(self, event, ...)
	if db.BossEmotesFrame and inCombat then
		return
	end
	return oldBE(self, event, ...)
end)

-----------------------------
-- Options
function plugin:GetOptions()	
	return LB.PLUGINS, L.NAME, self.GetGUI, [[Interface\CHATFRAME\UI-ChatIcon-Chat-Down]]
end

local function changeCallback()
	plugin:Reset()
end

function plugin:GetGUI()
	local self = self or plugin
	local UIF = LibStub("LibGUIFactory-1.0"):GetFactory("RW")
	local AceGUI = LibStub("AceGUI-3.0")
	
	local group, label, slider, check, igroup, button, color, select
	
	group = AceGUI:Create("ScrollFrame")
	group:SetLayout("Flow")
	
	label = UIF:MainTitleLabel(L.NAME)
	group:AddChild(label)
	
	igroup = UIF:InlineGroup("")
	group:AddChild(igroup)	
	
	label = UIF:NormalText(L.DESC)
	label:SetImage([[Interface\CHATFRAME\UI-ChatIcon-Chat-Down]])
	igroup:AddChild(label)
	
	igroup = UIF:InlineGroup("")
	group:AddChild(igroup)	
	
	check = UIF:CheckBox(LB.ENABLED, db, "PluginEnabled", function() 
		if not db.PluginEnabled then
			self:Disable()
		else
			self:Enable()
		end
	end)
	igroup:AddChild(check)
	
	label = UIF:NormalText(" ")
	igroup:AddChild(label)
	
	check = UIF:CheckBox(L.AUTO_WHISPER, db, "WhisperReply", function(widget, event, value)
		if value then 
			plugin:RegisterEvent("CHAT_MSG_WHISPER")
		else
			plugin:UnregisterEvent("CHAT_MSG_WHISPER")
		end
	end)
	igroup:AddChild(check)
	
	check = UIF:CheckBox(L.FILTER_AUTO_REPLY, db, "FilterOutIn", function(widget, event, value)
		if value then 
			ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", filterWhisperIn)
			ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", filterWhisperOut)
		else
			ChatFrame_RemoveMessageEventFilter("CHAT_MSG_WHISPER", filterWhisperIn)
			ChatFrame_RemoveMessageEventFilter("CHAT_MSG_WHISPER_INFORM", filterWhisperOut)
		end
	end)
	igroup:AddChild(check)
	
	check = UIF:CheckBox(L.FILTER_KNOW_BOSS_MODS, db, "BossModBlock", function(widget, event, value)
		if value then 
			ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID_WARNING", filterRaidWarning)
		else
			ChatFrame_RemoveMessageEventFilter("CHAT_MSG_RAID_WARNING", filterRaidWarning)
		end
	end)
	igroup:AddChild(check)
	
	label = UIF:NormalText(" ")
	igroup:AddChild(label)
	
	check = UIF:CheckBox(L.FILTER_EMOTES_FRAME, db, "BossEmotesFrame")
	igroup:AddChild(check)
	
	check = UIF:CheckBox(L.FILTER_EMOTES_CHAT, db, "BossEmotesChat", function(widget, event, value)
		if value then
			ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID_BOSS_EMOTE", filterBossEmote)
		else
			ChatFrame_RemoveMessageEventFilter("CHAT_MSG_RAID_BOSS_EMOTE", filterBossEmote)
		end
	end)
	igroup:AddChild(check)
		
	button = UIF:Button(LB.RESET, function() 
		plugin.db:ResetProfile() 
		plugin:Remove()
		plugin:Apply()
	end)
	button:SetUserData("confirm", true)
	button:SetUserData("TriggerGlobalUpdate", true)
	group:AddChild(button)
	
	group:FixScroll()
	
	return group
end